@@ -17,21 +17,21 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <qtest.h>
-
 #include "jobtest.h"
 
+#include <qtest.h>
+
 #include <klocalizedstring.h>
 
 #include <QDebug>
 #include <QPointer>
 #include <QSignalSpy>
-#include <QtCore/QFileInfo>
-#include <QtCore/QEventLoop>
-#include <QtCore/QDir>
-#include <QtCore/QHash>
-#include <QtCore/QVariant>
-#include <QtCore/QBuffer>
+#include <QFileInfo>
+#include <QEventLoop>
+#include <QDir>
+#include <QHash>
+#include <QVariant>
+#include <QBuffer>
 #include <QTemporaryFile>
 #include <QUrl>
 
@@ -78,7 +78,7 @@ static QString realSystemPath()
 
 void JobTest::initTestCase()
 {
-    QStandardPaths::enableTestMode(true);
+    QStandardPaths::setTestModeEnabled(true);
     QCoreApplication::instance()->setApplicationName("kio/jobtest"); // testing for #357499
 
     // To avoid a runtime dependency on klauncher
@@ -297,6 +297,42 @@ void JobTest::storedPutIODeviceTempFile()
     QVERIFY(QFile::remove(dest));
 }
 
+void JobTest::storedPutIODeviceFastDevice()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    const QUrl u = QUrl::fromLocalFile(filePath);
+    const QByteArray putDataContents = "This is the put data";
+    QBuffer putDataBuffer;
+    QVERIFY(putDataBuffer.open(QIODevice::ReadWrite));
+
+    KIO::StoredTransferJob *job = KIO::storedPut(&putDataBuffer, u, 0600, KIO::Overwrite | KIO::HideProgressInfo);
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*,ulong)));
+    QVERIFY(spyPercent.isValid());
+    QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setTotalSize(putDataContents.size());
+    job->setUiDelegate(nullptr);
+    job->setAsyncDataEnabled(true);
+
+    // Emit the readChannelFinished even before the job has had time to start
+    const auto pos = putDataBuffer.pos();
+    int size = putDataBuffer.write(putDataContents);
+    putDataBuffer.seek(pos);
+    putDataBuffer.readChannelFinished();
+
+    QVERIFY(job->exec());
+    QCOMPARE(size, putDataContents.size());
+    QCOMPARE(putDataBuffer.bytesAvailable(), 0);
+
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), (long long)putDataContents.size());
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+    QVERIFY(!spyPercent.isEmpty());
+}
+
 void JobTest::storedPutIODeviceSlowDevice()
 {
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -315,16 +351,61 @@ void JobTest::storedPutIODeviceSlowDevice()
     job->setUiDelegate(nullptr);
     job->setAsyncDataEnabled(true);
 
-    QTimer t;
-    t.setSingleShot(true);
-    int size = -1;
-    connect(&t, &QTimer::timeout, this, [&putDataBuffer, &size, putDataContents]() {
-        putDataBuffer.write(putDataContents);
-        putDataBuffer.seek(0);
-//         qDebug() << "written" << putDataBuffer.size();
-        size = putDataBuffer.bytesAvailable();
-    });
-    t.start(200);
+    int size = 0;
+    const auto writeOnce = [&putDataBuffer, &size, putDataContents]() {
+        const auto pos = putDataBuffer.pos();
+        size += putDataBuffer.write(putDataContents);
+        putDataBuffer.seek(pos);
+//         qDebug() << "written" << size;
+    };
+
+    QTimer::singleShot(200, this, writeOnce);
+    QTimer::singleShot(400, this, writeOnce);
+    // Simulate the transfer is done
+    QTimer::singleShot(450, this, [&putDataBuffer](){ putDataBuffer.readChannelFinished(); });
+
+    QVERIFY(job->exec());
+    QCOMPARE(size, putDataContents.size() * 2);
+    QCOMPARE(putDataBuffer.bytesAvailable(), 0);
+
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), (long long)putDataContents.size() * 2);
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+    QVERIFY(!spyPercent.isEmpty());
+}
+
+void JobTest::storedPutIODeviceSlowDeviceBigChunk()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    const QUrl u = QUrl::fromLocalFile(filePath);
+    const QByteArray putDataContents(300000, 'K'); // Make sure the 300000 is bigger than MAX_READ_BUF_SIZE
+    QBuffer putDataBuffer;
+    QVERIFY(putDataBuffer.open(QIODevice::ReadWrite));
+
+    KIO::StoredTransferJob *job = KIO::storedPut(&putDataBuffer, u, 0600, KIO::Overwrite | KIO::HideProgressInfo);
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*,ulong)));
+    QVERIFY(spyPercent.isValid());
+    QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setTotalSize(putDataContents.size());
+    job->setUiDelegate(nullptr);
+    job->setAsyncDataEnabled(true);
+
+    int size = 0;
+    const auto writeOnce = [&putDataBuffer, &size, putDataContents]() {
+        const auto pos = putDataBuffer.pos();
+        size += putDataBuffer.write(putDataContents);
+        putDataBuffer.seek(pos);
+//         qDebug() << "written" << size;
+    };
+
+    QTimer::singleShot(200, this, writeOnce);
+    // Simulate the transfer is done
+    QTimer::singleShot(450, this, [&putDataBuffer](){ putDataBuffer.readChannelFinished(); });
+
     QVERIFY(job->exec());
     QCOMPARE(size, putDataContents.size());
     QCOMPARE(putDataBuffer.bytesAvailable(), 0);
@@ -337,6 +418,31 @@ void JobTest::storedPutIODeviceSlowDevice()
     QVERIFY(!spyPercent.isEmpty());
 }
 
+void JobTest::asyncStoredPutReadyReadAfterFinish()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    const QUrl u = QUrl::fromLocalFile(filePath);
+
+    QBuffer putDataBuffer;
+    QVERIFY(putDataBuffer.open(QIODevice::ReadWrite));
+
+    KIO::StoredTransferJob *job = KIO::storedPut(&putDataBuffer, u, 0600, KIO::Overwrite | KIO::HideProgressInfo);
+    job->setAsyncDataEnabled(true);
+
+    bool jobFinished = false;
+
+    connect(job, &KJob::finished, [&jobFinished, &putDataBuffer] {
+        putDataBuffer.readyRead();
+        jobFinished = true;
+    });
+
+    QTimer::singleShot(200, this, [job]() {
+        job->kill();
+    });
+
+    QTRY_VERIFY(jobFinished);
+}
+
 ////
 
 void JobTest::copyLocalFile(const QString &src, const QString &dest)
@@ -497,7 +603,7 @@ static void copyLocalSymlink(const QString &src, const QString &dest, const QStr
     KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
     job->setUiDelegate(nullptr);
     job->setUiDelegateExtension(nullptr);
-    QVERIFY2(job->exec(), qPrintable(job->error()));
+    QVERIFY2(job->exec(), qPrintable(QString::number(job->error())));
     QVERIFY(QT_LSTAT(QFile::encodeName(dest).constData(), &buf) == 0); // dest exists
     QCOMPARE(linkTarget(dest), expectedLinkTarget);
 
@@ -581,6 +687,9 @@ void JobTest::copyAbsoluteSymlinkToOtherPartition()
 
 void JobTest::copyFolderWithUnaccessibleSubfolder()
 {
+#ifdef Q_OS_WIN
+    QSKIP("Skipping unaccessible folder test on Windows, cannot remove all permissions from a folder");
+#endif
     const QString src_dir = homeTmpDir() + "srcHome";
     const QString dst_dir = homeTmpDir() + "dstHome";
 
@@ -814,6 +923,9 @@ void JobTest::moveDirectoryToOtherPartition()
 
 void JobTest::moveFileNoPermissions()
 {
+#ifdef Q_OS_WIN
+    QSKIP("Skipping unaccessible folder test on Windows, cannot remove all permissions from a folder");
+#endif
     // Given a file that cannot be moved (subdir has no permissions)
     const QString subdir = homeTmpDir() + "subdir";
     QVERIFY(QDir().mkpath(subdir));
@@ -845,6 +957,9 @@ void JobTest::moveFileNoPermissions()
 
 void JobTest::moveDirectoryNoPermissions()
 {
+#ifdef Q_OS_WIN
+    QSKIP("Skipping unaccessible folder test on Windows, cannot remove all permissions from a folder");
+#endif
     // Given a dir that cannot be moved (parent dir has no permissions)
     const QString subdir = homeTmpDir() + "subdir";
     const QString src = subdir + "/thedir";
@@ -889,17 +1004,14 @@ void JobTest::listRecursive()
     m_names.sort();
     QByteArray ref_names = QByteArray(".,..,"
                                       "dirFromHome,dirFromHome/testfile,"
-#ifndef Q_OS_WIN
-                                      "dirFromHome/testlink,"
-#endif
+                                      "dirFromHome/testlink," // exists on Windows too, see createTestDirectory
                                       "dirFromHome_copied,"
                                       "dirFromHome_copied/dirFromHome,dirFromHome_copied/dirFromHome/testfile,"
-#ifndef Q_OS_WIN
                                       "dirFromHome_copied/dirFromHome/testlink,"
-#endif
                                       "dirFromHome_copied/testfile,"
+                                      "dirFromHome_copied/testlink,"
 #ifndef Q_OS_WIN
-                                      "dirFromHome_copied/testlink,dirFromHome_link,"
+                                      "dirFromHome_link,"
 #endif
                                       "fileFromHome");
 
@@ -984,7 +1096,7 @@ void JobTest::directorySize()
 #else
     QCOMPARE(job->totalFiles(), 7ULL); // see expected result in listRecursive() above
     QCOMPARE(job->totalSubdirs(), 4ULL); // see expected result in listRecursive() above
-    QVERIFY(job->totalSize() >= 260); // size of subdir entries is filesystem dependent. E.g. this is 16428 with ext4 but only 272 with xfs.
+    QVERIFY2(job->totalSize() >= 60, qPrintable(QString("totalSize was %1").arg(job->totalSize()))); // size of subdir entries is filesystem dependent. E.g. this is 16428 with ext4 but only 272 with xfs, and 63 on FreeBSD
 #endif
 
     qApp->sendPostedEvents(nullptr, QEvent::DeferredDelete);
@@ -1362,7 +1474,7 @@ void JobTest::statSymlink()
     const QString symlink = otherTmpDir() + "link";
     QVERIFY(QFile(filePath).link(symlink));
     QVERIFY(QFile::exists(symlink));
-    setTimeStamp(symlink, QDateTime::currentDateTime().addSecs(-20)); // differenciate link time and source file time
+    setTimeStamp(symlink, QDateTime::currentDateTime().addSecs(-20)); // differentiate link time and source file time
 
     const QUrl url(QUrl::fromLocalFile(symlink));
     KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
@@ -1422,6 +1534,26 @@ void JobTest::chmodFile()
     QFile::remove(filePath);
 }
 
+#ifdef Q_OS_UNIX
+void JobTest::chmodSticky()
+{
+    const QString dirPath = homeTmpDir() + "dirForChmodSticky";
+    QDir().mkpath(dirPath);
+    KFileItem item(QUrl::fromLocalFile(dirPath));
+    const mode_t origPerm = item.permissions();
+    mode_t newPerm = origPerm ^ S_ISVTX;
+    QVERIFY(newPerm != origPerm);
+    KFileItemList items({item});
+    KIO::Job *job = KIO::chmod(items, newPerm, S_ISVTX, QString(), QString(), false, KIO::HideProgressInfo);
+    job->setUiDelegate(nullptr);
+    QVERIFY(job->exec());
+
+    KFileItem newItem(QUrl::fromLocalFile(dirPath));
+    QCOMPARE(QString::number(newItem.permissions(), 8), QString::number(newPerm, 8));
+    QVERIFY(QDir().rmdir(dirPath));
+}
+#endif
+
 void JobTest::chmodFileError()
 {
     // chown(root) should fail
