@@ -196,26 +196,27 @@ void KFileItemPrivate::init()
             const QString path = m_url.adjusted(QUrl::StripTrailingSlash).toLocalFile();
             const QByteArray pathBA = QFile::encodeName(path);
             if (QT_LSTAT(pathBA.constData(), &buf) == 0) {
-                m_entry.insert(KIO::UDSEntry::UDS_DEVICE_ID,           buf.st_dev);
-                m_entry.insert(KIO::UDSEntry::UDS_INODE,               buf.st_ino);
+                m_entry.reserve(9);
+                m_entry.replace(KIO::UDSEntry::UDS_DEVICE_ID,           buf.st_dev);
+                m_entry.replace(KIO::UDSEntry::UDS_INODE,               buf.st_ino);
 
                 mode_t mode = buf.st_mode;
                 if ((buf.st_mode & QT_STAT_MASK) == QT_STAT_LNK) {
                     m_bLink = true;
-                    if (QT_STAT(pathBA, &buf) == 0) {
+                    if (QT_STAT(pathBA.constData(), &buf) == 0) {
                         mode = buf.st_mode;
                     } else {// link pointing to nowhere (see FileProtocol::createUDSEntry() in ioslaves/file/file.cpp)
                         mode = (QT_STAT_MASK - 1) | S_IRWXU | S_IRWXG | S_IRWXO;
                     }
                 }
-                m_entry.insert(KIO::UDSEntry::UDS_SIZE,      buf.st_size);
-                m_entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, buf.st_mode & QT_STAT_MASK); // extract file type
-                m_entry.insert(KIO::UDSEntry::UDS_ACCESS,    buf.st_mode & 07777); // extract permissions
-                m_entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME,   buf.st_mtime); // TODO: we could use msecs too...
-                m_entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME,         buf.st_atime);
+                m_entry.replace(KIO::UDSEntry::UDS_SIZE,      buf.st_size);
+                m_entry.replace(KIO::UDSEntry::UDS_FILE_TYPE, buf.st_mode & QT_STAT_MASK); // extract file type
+                m_entry.replace(KIO::UDSEntry::UDS_ACCESS,    buf.st_mode & 07777); // extract permissions
+                m_entry.replace(KIO::UDSEntry::UDS_MODIFICATION_TIME,   buf.st_mtime); // TODO: we could use msecs too...
+                m_entry.replace(KIO::UDSEntry::UDS_ACCESS_TIME,         buf.st_atime);
 #ifndef Q_OS_WIN
-                m_entry.insert(KIO::UDSEntry::UDS_USER,                KUser(buf.st_uid).loginName());
-                m_entry.insert(KIO::UDSEntry::UDS_GROUP,               KUserGroup(buf.st_gid).name());
+                m_entry.replace(KIO::UDSEntry::UDS_USER,                KUser(buf.st_uid).loginName());
+                m_entry.replace(KIO::UDSEntry::UDS_GROUP,               KUserGroup(buf.st_gid).name());
 #endif
 
                 // TODO: these can be removed, we can use UDS_FILE_TYPE and UDS_ACCESS everywhere
@@ -303,7 +304,7 @@ static uint udsFieldForTime(KFileItem::FileTimes mappedWhich)
 
 void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, uint time_t_val) const
 {
-    m_entry.insert(udsFieldForTime(mappedWhich), time_t_val);
+    m_entry.replace(udsFieldForTime(mappedWhich), time_t_val);
 }
 
 void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, const QDateTime &val) const
@@ -547,7 +548,7 @@ void KFileItem::setLocalPath(const QString &path)
         return;
     }
 
-    d->m_entry.insert(KIO::UDSEntry::UDS_LOCAL_PATH, path);
+    d->m_entry.replace(KIO::UDSEntry::UDS_LOCAL_PATH, path);
 }
 
 void KFileItem::setName(const QString &name)
@@ -562,7 +563,7 @@ void KFileItem::setName(const QString &name)
         d->m_strText = KIO::decodeFileName(d->m_strName);
     }
     if (d->m_entry.contains(KIO::UDSEntry::UDS_NAME)) {
-        d->m_entry.insert(KIO::UDSEntry::UDS_NAME, d->m_strName);    // #195385
+        d->m_entry.replace(KIO::UDSEntry::UDS_NAME, d->m_strName);    // #195385
     }
 
 }
@@ -812,9 +813,9 @@ QString KFileItem::mimeComment() const
     }
 
     // Support for .directory file in directories
-    if (isLocalUrl && isDir() && isDirectoryMounted(url)) {
+    if (isLocalUrl && isDir() && !d->isSlow() && isDirectoryMounted(url)) {
         QUrl u(url);
-        u.setPath(concatPaths(u.path(), QLatin1String(".directory")));
+        u.setPath(concatPaths(u.path(), QStringLiteral(".directory")));
         const KDesktopFile cfg(u.toLocalFile());
         const QString comment = cfg.readComment();
         if (!comment.isEmpty()) {
@@ -861,7 +862,7 @@ static QString iconFromDirectoryFile(const QString &path)
     if (icon.startsWith(QLatin1String("./"))) {
         // path is relative with respect to the location
         // of the .directory file (#73463)
-        return path + icon.mid(1);
+        return path + icon.midRef(1);
     }
     return icon;
 }
@@ -873,7 +874,6 @@ static QString iconFromDesktopFile(const QString &path)
     if (cfg.hasLinkType()) {
         const KConfigGroup group = cfg.desktopGroup();
         const QString emptyIcon = group.readEntry("EmptyIcon");
-        const QString type = cfg.readPath();
         if (!emptyIcon.isEmpty()) {
             const QString u = cfg.readUrl();
             const QUrl url(u);
@@ -983,14 +983,14 @@ QStringList KFileItem::overlays() const
         return QStringList();
     }
 
-    QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(',',QString::SkipEmptyParts);
+    QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(QLatin1Char(','), QString::SkipEmptyParts);
 
     if (d->m_bLink) {
         names.append(QStringLiteral("emblem-symbolic-link"));
     }
 
     if (!isReadable()) {
-        names.append(QStringLiteral("object-locked"));
+        names.append(QStringLiteral("emblem-locked"));
     }
 
     if (checkDesktopFile(*this, false)) {
@@ -1018,11 +1018,15 @@ QStringList KFileItem::overlays() const
         names.append(QStringLiteral("hidden"));
     }
 #ifndef Q_OS_WIN
-    if (((d->m_fileMode & QT_STAT_MASK) == QT_STAT_DIR) && d->m_bIsLocalUrl) {
-        if (KSambaShare::instance()->isDirectoryShared(d->m_url.toLocalFile()) ||
-                KNFSShare::instance()->isDirectoryShared(d->m_url.toLocalFile())) {
-            //qDebug() << d->m_url.path();
-            names.append(QStringLiteral("emblem-shared"));
+    if (isDir()) {
+        bool isLocalUrl;
+        const QUrl url = mostLocalUrl(&isLocalUrl);
+        if (isLocalUrl) {
+            const QString path = url.toLocalFile();
+            if (KSambaShare::instance()->isDirectoryShared(path)
+                    || KNFSShare::instance()->isDirectoryShared(path)) {
+                names.append(QStringLiteral("emblem-shared"));
+            }
         }
     }
 #endif  // Q_OS_WIN
@@ -1119,7 +1123,7 @@ bool KFileItem::isHidden() const
     if (fileName.isEmpty()) { // e.g. "trash:/"
         fileName = d->m_strName;
     }
-    return fileName.length() > 1 && fileName[0] == '.';  // Just "." is current directory, not hidden.
+    return fileName.length() > 1 && fileName[0] == QLatin1Char('.');  // Just "." is current directory, not hidden.
 }
 
 void KFileItem::setHidden()
@@ -1188,7 +1192,7 @@ QString KFileItem::getStatusBarInfo() const
     const QString comment = mimeComment();
 
     if (d->m_bLink) {
-        text += ' ';
+        text += QLatin1Char(' ');
         if (comment.isEmpty()) {
             text += i18n("(Symbolic Link to %1)", linkDest());
         } else {
@@ -1235,6 +1239,25 @@ bool KFileItem::operator!=(const KFileItem &other) const
     return !operator==(other);
 }
 
+bool KFileItem::operator<(const KFileItem &other) const
+{
+    if (!other.d) {
+        return false;
+    }
+    if (!d) {
+        return other.d->m_url.isValid();
+    }
+    return d->m_url < other.d->m_url;
+}
+
+bool KFileItem::operator<(const QUrl &other) const
+{
+    if (!d) {
+        return other.isValid();
+    }
+    return d->m_url < other;
+}
+
 KFileItem::operator QVariant() const
 {
     return qVariantFromValue(*this);
