@@ -21,6 +21,9 @@
  *
  **/
 
+// TODO: remove me
+#undef QT_NO_CAST_FROM_ASCII
+
 #include "slavebase.h"
 
 #include <config-kiocore.h>
@@ -32,15 +35,16 @@
 #include <process.h>
 #endif
 
-#include <QtCore/QFile>
-#include <QtCore/QList>
-#include <QtCore/QDateTime>
-#include <QtCore/QElapsedTimer>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QDataStream>
+#include <QFile>
+#include <QList>
+#include <QDateTime>
+#include <QElapsedTimer>
+#include <QCoreApplication>
+#include <QDataStream>
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
+#include <kcrash.h>
 #include <klocalizedstring.h>
 
 #include "kremoteencoding.h"
@@ -53,14 +57,11 @@
 #include "kpasswdserverclient.h"
 #include "kiocoredebug.h"
 
-#ifndef NDEBUG
-#if HAVE_BACKTRACE
-#include <execinfo.h>
-#endif
+#ifdef Q_OS_UNIX
+#include <KAuth>
 #endif
 
 extern "C" {
-    Q_NORETURN static void sigsegv_handler(int sig);
     static void sigpipe_handler(int sig);
 }
 
@@ -79,11 +80,16 @@ namespace KIO
 class SlaveBasePrivate
 {
 public:
-    SlaveBase *q;
-    SlaveBasePrivate(SlaveBase *owner): q(owner), nextTimeoutMsecs(0), m_passwdServerClient(nullptr)
+    SlaveBase * const q;
+    explicit SlaveBasePrivate(SlaveBase *owner)
+        : q(owner)
+        , nextTimeoutMsecs(0)
+        , m_passwdServerClient(nullptr)
+        , m_confirmationAsked(false)
+        , m_privilegeOperationStatus(OperationNotAllowed)
     {
         if (!qEnvironmentVariableIsEmpty("KIOSLAVE_ENABLE_TESTMODE")) {
-            QStandardPaths::enableTestMode(true);
+            QStandardPaths::setTestModeEnabled(true);
         }
         pendingListEntries.reserve(KIO_MAX_ENTRIES_PER_BATCH);
     }
@@ -121,6 +127,45 @@ public:
     KPasswdServerClient *m_passwdServerClient;
     bool m_rootEntryListed = false;
 
+    bool m_confirmationAsked;
+    QSet<QString> m_tempAuths;
+    QString m_warningCaption;
+    QString m_warningMessage;
+    int m_privilegeOperationStatus;
+
+    PrivilegeOperationStatus askConfirmation()
+    {
+        int status = q->messageBox(SlaveBase::WarningContinueCancel, m_warningMessage, m_warningCaption, QStringLiteral("Continue"), QStringLiteral("Cancel"));
+        switch (status) {
+        case SlaveBase::Continue:
+            return OperationAllowed;
+        case SlaveBase::Cancel:
+            return OperationCanceled;
+        default:
+            return OperationNotAllowed;
+        }
+    }
+
+    void updateTempAuthStatus()
+    {
+#ifdef Q_OS_UNIX
+        QSet<QString>::iterator it = m_tempAuths.begin();
+        while (it != m_tempAuths.end()) {
+            KAuth::Action action(*it);
+            if (action.status() != KAuth::Action::AuthorizedStatus) {
+                it = m_tempAuths.erase(it);
+            } else {
+                ++it;
+            }
+        }
+#endif
+    }
+
+    bool hasTempAuth() const
+    {
+        return !m_tempAuths.isEmpty();
+    }
+
     // Reconstructs configGroup from configData and mIncomingMetaData
     void rebuildConfig()
     {
@@ -202,32 +247,10 @@ SlaveBase::SlaveBase(const QByteArray &protocol,
     Q_ASSERT(!app_socket.isEmpty());
     d->poolSocket = QFile::decodeName(pool_socket);
     s_protocol = protocol.data();
-#ifdef Q_OS_UNIX
-    if (qEnvironmentVariableIsEmpty("KDE_DEBUG")) {
-        ::signal(SIGSEGV, &sigsegv_handler);
-        ::signal(SIGILL, &sigsegv_handler);
-        ::signal(SIGTRAP, &sigsegv_handler);
-        ::signal(SIGABRT, &sigsegv_handler);
-        ::signal(SIGBUS, &sigsegv_handler);
-        ::signal(SIGALRM, &sigsegv_handler);
-        ::signal(SIGFPE, &sigsegv_handler);
-#ifdef SIGPOLL
-        ::signal(SIGPOLL, &sigsegv_handler);
-#endif
-#ifdef SIGSYS
-        ::signal(SIGSYS, &sigsegv_handler);
-#endif
-#ifdef SIGVTALRM
-        ::signal(SIGVTALRM, &sigsegv_handler);
-#endif
-#ifdef SIGXCPU
-        ::signal(SIGXCPU, &sigsegv_handler);
-#endif
-#ifdef SIGXFSZ
-        ::signal(SIGXFSZ, &sigsegv_handler);
-#endif
-    }
 
+    KCrash::initialize();
+
+#ifdef Q_OS_UNIX
     struct sigaction act;
     act.sa_handler = sigpipe_handler;
     sigemptyset(&act.sa_mask);
@@ -244,8 +267,7 @@ SlaveBase::SlaveBase(const QByteArray &protocol,
     d->isConnectedToApp = true;
 
     // by kahl for netmgr (need a way to identify slaves)
-    d->slaveid = protocol;
-    d->slaveid += QString::number(getpid());
+    d->slaveid = QString::fromUtf8(protocol) + QString::number(getpid());
     d->resume = false;
     d->needSendCanResume = false;
     d->config = new KConfig(QString(), KConfig::SimpleConfig);
@@ -312,6 +334,7 @@ void SlaveBase::dispatchLoop()
                 disconnectSlave();
                 d->isConnectedToApp = false;
                 closeConnection();
+                d->updateTempAuthStatus();
                 connectSlave(d->poolSocket);
             } else {
                 break;
@@ -407,7 +430,7 @@ KRemoteEncoding *SlaveBase::remoteEncoding()
     }
 
     const QByteArray charset(metaData(QStringLiteral("Charset")).toLatin1());
-    return (d->remotefile = new KRemoteEncoding(charset));
+    return (d->remotefile = new KRemoteEncoding(charset.constData()));
 }
 
 void SlaveBase::data(const QByteArray &data)
@@ -452,6 +475,8 @@ void SlaveBase::error(int _errid, const QString &_text)
     //reset
     d->totalSize = 0;
     d->inOpenLoop = false;
+    d->m_confirmationAsked = false;
+    d->m_privilegeOperationStatus = OperationNotAllowed;
 }
 
 void SlaveBase::connected()
@@ -465,10 +490,10 @@ void SlaveBase::finished()
         if (!d->m_rootEntryListed) {
             qCWarning(KIO_CORE) << "UDSEntry for '.' not found, creating a default one. Please fix the" << QCoreApplication::applicationName() << "KIO slave";
             KIO::UDSEntry entry;
-            entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
-            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-            entry.insert(KIO::UDSEntry::UDS_SIZE, 0);
-            entry.insert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IXOTH);
+            entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
+            entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+            entry.fastInsert(KIO::UDSEntry::UDS_SIZE, 0);
+            entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IXOTH);
             d->pendingListEntries.append(entry);
         }
 
@@ -494,6 +519,8 @@ void SlaveBase::finished()
     d->totalSize = 0;
     d->inOpenLoop = false;
     d->m_rootEntryListed = false;
+    d->m_confirmationAsked = false;
+    d->m_privilegeOperationStatus = OperationNotAllowed;
 }
 
 void SlaveBase::needSubUrlData()
@@ -505,11 +532,8 @@ void SlaveBase::slaveStatus(const QString &host, bool connected)
 {
     qint64 pid = getpid();
     qint8 b = connected ? 1 : 0;
-    KIO_DATA << pid << mProtocol << host << b;
-    if (d->onHold) {
-        stream << d->onHoldUrl;
-    }
-    send(MSG_SLAVE_STATUS, data);
+    KIO_DATA << pid << mProtocol << host << b << d->onHold << d->onHoldUrl << d->hasTempAuth();
+    send(MSG_SLAVE_STATUS_V2, data);
 }
 
 void SlaveBase::canResume()
@@ -738,33 +762,6 @@ void SlaveBase::listEntries(const UDSEntryList &list)
     send(MSG_LIST_ENTRIES, data);
 }
 
-Q_NORETURN static void sigsegv_handler(int sig)
-{
-#ifdef Q_OS_UNIX
-    ::signal(sig, SIG_DFL); // Next one kills
-
-    //Kill us if we deadlock
-    ::signal(SIGALRM, SIG_DFL);
-    alarm(5);  //generate an alarm signal in 5 seconds, in this time the slave has to exit
-
-    // Debug and printf should be avoided because they might
-    // call malloc.. and get in a nice recursive malloc loop
-    char buffer[120];
-    qsnprintf(buffer, sizeof(buffer), "kioslave: ####### CRASH ###### protocol = %s pid = %d signal = %d\n", s_protocol, getpid(), sig);
-    write(2, buffer, strlen(buffer));
-#ifndef NDEBUG
-#if HAVE_BACKTRACE
-    void *trace[256];
-    int n = backtrace(trace, 256);
-    if (n) {
-        backtrace_symbols_fd(trace, n, 2);
-    }
-#endif
-#endif
-    ::exit(1);
-#endif
-}
-
 static void sigpipe_handler(int)
 {
     // We ignore a SIGPIPE in slaves.
@@ -824,11 +821,11 @@ KIOCORE_EXPORT QString KIO::unsupportedActionErrorString(const QString &protocol
     }/*end switch*/
 }
 
-void SlaveBase::openConnection(void)
+void SlaveBase::openConnection()
 {
     error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CONNECT));
 }
-void SlaveBase::closeConnection(void)
+void SlaveBase::closeConnection()
 { } // No response!
 void SlaveBase::stat(QUrl const &)
 {
@@ -1296,7 +1293,7 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
     } break;
     default: {
         // Some command we don't understand.
-        // Just ignore it, it may come from some future version of KDE.
+        // Just ignore it, it may come from some future version of KIO.
     } break;
     }
 }
@@ -1336,7 +1333,7 @@ void SlaveBase::dispatchOpenCommand(int command, const QByteArray &data)
         break;
     default:
         // Some command we don't understand.
-        // Just ignore it, it may come from some future version of KDE.
+        // Just ignore it, it may come from some future version of KIO.
         break;
     }
 }
@@ -1461,8 +1458,25 @@ int SlaveBase::waitForHostInfo(QHostInfo &info)
 
 PrivilegeOperationStatus SlaveBase::requestPrivilegeOperation()
 {
-    QByteArray buffer;
-    send(MSG_PRIVILEGE_EXEC);
-    waitForAnswer(MSG_PRIVILEGE_EXEC, 0, buffer);
-    return KIO::PrivilegeOperationStatus(buffer.toInt());
+    if (d->m_privilegeOperationStatus == OperationNotAllowed) {
+        QByteArray buffer;
+        send(MSG_PRIVILEGE_EXEC);
+        waitForAnswer(MSG_PRIVILEGE_EXEC, 0, buffer);
+        QDataStream ds(buffer);
+        ds >> d->m_privilegeOperationStatus >> d->m_warningCaption >> d-> m_warningMessage;
+    }
+
+    if (metaData(QStringLiteral("UnitTesting")) != QLatin1String("true") &&
+            d->m_privilegeOperationStatus == OperationAllowed &&
+            !d->m_confirmationAsked) {
+        d->m_privilegeOperationStatus = d->askConfirmation();
+        d->m_confirmationAsked = true;
+    }
+
+    return KIO::PrivilegeOperationStatus(d->m_privilegeOperationStatus);
+}
+
+void SlaveBase::addTemporaryAuthorization(const QString &action)
+{
+    d->m_tempAuths.insert(action);
 }
