@@ -87,7 +87,7 @@ void KAutoMountPrivate::slotResult(KJob *job)
             const QUrl url = QUrl::fromLocalFile(mp->mountPoint());
             //qDebug() << "KAutoMount: m_strDevice=" << m_strDevice << " -> mountpoint=" << mountpoint;
             if (m_bShowFilemanagerWindow) {
-                KRun::runUrl(url, QStringLiteral("inode/directory"), nullptr /*TODO - window*/);
+                KRun::runUrl(url, QStringLiteral("inode/directory"), nullptr /*TODO - window*/, KRun::RunFlags(KRun::RunExecutables));
             }
             // Notify about the new stuff in that dir, in case of opened windows showing it
             org::kde::KDirNotify::emitFilesAdded(url);
@@ -109,7 +109,7 @@ public:
     KAutoUnmountPrivate(KAutoUnmount *qq, const QString &_mountpoint, const QString &_desktopFile)
         : q(qq), m_desktopFile(_desktopFile), m_mountpoint(_mountpoint)
     {}
-    KAutoUnmount *q;
+    KAutoUnmount * const q;
     QString m_desktopFile;
     QString m_mountpoint;
 
