@@ -135,7 +135,7 @@ bool KRun::isExecutableFile(const QUrl &url, const QString &mimetype)
         QMimeType mimeType = db.mimeTypeForName(mimetype);
         if (mimeType.inherits(QStringLiteral("application/x-executable")) ||
 #ifdef Q_OS_WIN
-                mimeType.inherits(QLatin1String("application/x-ms-dos-executable")) ||
+                mimeType.inherits(QStringLiteral("application/x-ms-dos-executable")) ||
 #endif
                 mimeType.inherits(QStringLiteral("application/x-executable-script")) ||
                 mimeType.inherits(QStringLiteral("application/x-sharedlib"))
@@ -256,7 +256,7 @@ bool KRun::displayOpenWithDialog(const QList<QUrl> &lst, QWidget *window, bool t
     }
 
 #ifdef Q_OS_WIN
-    KConfigGroup cfgGroup(KSharedConfig::openConfig(), "KOpenWithDialog Settings");
+    KConfigGroup cfgGroup(KSharedConfig::openConfig(), QStringLiteral("KOpenWithDialog Settings"));
     if (cfgGroup.readEntry("Native", true)) {
         return KRun::KRunPrivate::displayNativeOpenWithDialog(lst, window, tempFiles,
                 suggestedFileName, asn);
@@ -282,7 +282,7 @@ void KRun::shellQuote(QString &_str)
     if (_str.isEmpty()) { // Don't create an explicit empty parameter
         return;
     }
-    QChar q('\'');
+    const QChar q = QLatin1Char('\'');
     _str.replace(q, QLatin1String("'\\''")).prepend(q).append(q);
 }
 #endif
@@ -454,12 +454,12 @@ static qint64 runApplicationImpl(const KService &_service, const QList<QUrl> &_u
     if (_service.runOnDiscreteGpu() && s_gpuCheck == NotChecked) {
         // Check whether we have a discrete gpu
         bool hasDiscreteGpu = false;
-        QDBusInterface iface(QLatin1String("org.kde.Solid.PowerManagement"),
-                             QLatin1String("/org/kde/Solid/PowerManagement"),
-                             QLatin1String("org.kde.Solid.PowerManagement"),
+        QDBusInterface iface(QStringLiteral("org.kde.Solid.PowerManagement"),
+                             QStringLiteral("/org/kde/Solid/PowerManagement"),
+                             QStringLiteral("org.kde.Solid.PowerManagement"),
                              QDBusConnection::sessionBus());
         if (iface.isValid()) {
-            QDBusReply<bool> reply = iface.call(QLatin1String("hasDualGpu"));
+            QDBusReply<bool> reply = iface.call(QStringLiteral("hasDualGpu"));
             if (reply.isValid()) {
                 hasDiscreteGpu = reply.value();
             }
@@ -469,7 +469,7 @@ static qint64 runApplicationImpl(const KService &_service, const QList<QUrl> &_u
     }
 
     if (_service.runOnDiscreteGpu() && s_gpuCheck == Present) {
-        proc->setEnv(QLatin1String("DRI_PRIME"), QLatin1String("1"));
+        proc->setEnv(QStringLiteral("DRI_PRIME"), QStringLiteral("1"));
     }
 
     QString path(_service.path());
@@ -700,8 +700,8 @@ static bool makeServiceExecutable(const KService &service, QWidget *window)
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Ok), KStandardGuiItem::cont());
     buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
     buttonBox->button(QDialogButtonBox::Cancel)->setFocus();
-    QObject::connect(buttonBox, SIGNAL(accepted()), baseDialog, SLOT(accept()));
-    QObject::connect(buttonBox, SIGNAL(rejected()), baseDialog, SLOT(reject()));
+    QObject::connect(buttonBox, &QDialogButtonBox::accepted, baseDialog, &QDialog::accept);
+    QObject::connect(buttonBox, &QDialogButtonBox::rejected, baseDialog, &QDialog::reject);
     topLayout->addWidget(buttonBox);
 
     // Constrain maximum size.  Minimum size set in
@@ -920,7 +920,7 @@ void KRun::KRunPrivate::init(const QUrl &url, QWidget *window,
     // Reason: We must complete the constructor before we do anything else.
     m_bCheckPrompt = false;
     m_bInit = true;
-    q->connect(m_timer, SIGNAL(timeout()), q, SLOT(slotTimeout()));
+    q->connect(m_timer, &QTimer::timeout, q, &KRun::slotTimeout);
     startTimer();
     //qDebug() << "new KRun" << q << url << "timer=" << m_timer;
 }
@@ -1033,8 +1033,8 @@ void KRun::init()
     KIO::JobFlags flags = d->m_bProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
     KIO::StatJob *job = KIO::stat(d->m_strURL, KIO::StatJob::SourceSide, 0 /* no details */, flags);
     KJobWidgets::setWindow(job, d->m_window);
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(slotStatResult(KJob*)));
+    connect(job, &KJob::result,
+            this, &KRun::slotStatResult);
     d->m_job = job;
     //qDebug() << "Job" << job << "is about stating" << d->m_strURL;
 }
@@ -1052,9 +1052,9 @@ bool KRun::KRunPrivate::runExecutable(const QString &_exec)
 {
     QList<QUrl> urls;
     urls.append(m_strURL);
-    if (_exec.startsWith('!')) {
-        QString exec = _exec.mid(1); // Literal command
-        exec += QLatin1String(" %u");
+    if (_exec.startsWith(QLatin1Char('!'))) {
+        // Literal command
+        const QString exec = _exec.midRef(1) + QLatin1String(" %u");
         if (q->run(exec, urls, m_window, QString(), QString(), m_asn)) {
             m_bFinished = true;
             startTimer();
@@ -1083,7 +1083,7 @@ void KRun::KRunPrivate::showPrompt()
 
 bool KRun::KRunPrivate::isPromptNeeded()
 {
-    if (m_strURL == QUrl("remote:/x-wizard_service.desktop")) {
+    if (m_strURL == QUrl(QStringLiteral("remote:/x-wizard_service.desktop"))) {
         return false;
     }
     const QMimeDatabase db;
@@ -1156,10 +1156,10 @@ void KRun::scanFile()
     KIO::JobFlags flags = d->m_bProgressInfo ? KIO::DefaultFlags : KIO::HideProgressInfo;
     KIO::TransferJob *job = KIO::get(d->m_strURL, KIO::NoReload /*reload*/, flags);
     KJobWidgets::setWindow(job, d->m_window);
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(slotScanFinished(KJob*)));
-    connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
-            this, SLOT(slotScanMimeType(KIO::Job*,QString)));
+    connect(job, &KJob::result,
+            this, &KRun::slotScanFinished);
+    connect(job, QOverload<KIO::Job*,const QString&>::of(&KIO::TransferJob::mimetype),
+            this, &KRun::slotScanMimeType);
     d->m_job = job;
     //qDebug() << "Job" << job << "is about getting from" << d->m_strURL;
 }
@@ -1349,7 +1349,11 @@ void KRun::foundMimeType(const QString &type)
         d->m_strURL = QUrl::fromLocalFile(d->m_localPath);
     }
 
-    if (!KRun::runUrl(d->m_strURL, type, d->m_window, false /*tempfile*/, d->m_runExecutables, d->m_suggestedFileName, d->m_asn)) {
+    KRun::RunFlags runFlags;
+    if (d->m_runExecutables) {
+        runFlags |= KRun::RunExecutables;
+    }
+    if (!KRun::runUrl(d->m_strURL, type, d->m_window, runFlags, d->m_suggestedFileName, d->m_asn)) {
         d->m_bFault = true;
     }
     setFinished(true);
@@ -1562,8 +1566,8 @@ KProcessRunner::KProcessRunner(KProcess *p, const QString &executable, const KSt
     m_pid = 0;
     process = p;
     m_executable = executable;
-    connect(process, SIGNAL(finished(int,QProcess::ExitStatus)),
-            this, SLOT(slotProcessExited(int,QProcess::ExitStatus)));
+    connect(process, QOverload<int,QProcess::ExitStatus>::of(&QProcess::finished),
+            this, &KProcessRunner::slotProcessExited);
 
     process->start();
     if (!process->waitForStarted()) {
