@@ -28,6 +28,7 @@
 #include "kprotocolmanager.h"
 #include "../pathhelpers_p.h"
 
+#include <kdirnotify.h>
 #include <kjobwidgets.h>
 #include <klocalizedstring.h>
 #include <kmessagebox.h>
@@ -53,7 +54,7 @@
  * @param mimeData the mimeData to paste, usually QApplication::clipboard()->mimeData()
  * @param destDir Destination directory where the items will be copied/moved.
  * @param flags the flags are passed to KIO::copy or KIO::move.
- * @return the copy or move job handling the operation, or 0 if there is nothing to do
+ * @return the copy or move job handling the operation, or @c nullptr if there is nothing to do
  * @since ...
  */
 //KIOWIDGETS_EXPORT Job *pasteClipboardUrls(const QUrl& destDir, JobFlags flags = DefaultFlags);
@@ -73,6 +74,37 @@ static KIO::Job *pasteClipboardUrls(const QMimeData *mimeData, const QUrl &destD
     return nullptr;
 }
 
+static QUrl getDestinationUrl(const QUrl &srcUrl, const QUrl &destUrl, QWidget *widget)
+{
+    KIO::StatJob *job = KIO::stat(destUrl, destUrl.isLocalFile() ? KIO::HideProgressInfo : KIO::DefaultFlags);
+    job->setDetails(0);
+    job->setSide(KIO::StatJob::DestinationSide);
+    KJobWidgets::setWindow(job, widget);
+
+    // Check for existing destination file.
+    // When we were using CopyJob, we couldn't let it do that (would expose
+    // an ugly tempfile name as the source URL)
+    // And now we're using a put job anyway, no destination checking included.
+    if (job->exec()) {
+        KIO::RenameDialog dlg(widget,
+                              i18n("File Already Exists"),
+                              srcUrl,
+                              destUrl,
+                              KIO::RenameDialog_Overwrite);
+        KIO::RenameDialog_Result res = static_cast<KIO::RenameDialog_Result>(dlg.exec());
+
+        if (res == KIO::Result_Rename) {
+            return dlg.newDestUrl();
+        } else if (res == KIO::Result_Cancel) {
+            return QUrl();
+        } else if (res == KIO::Result_Overwrite) {
+            return destUrl;
+        }
+    }
+
+    return destUrl;
+}
+
 static QUrl getNewFileName(const QUrl &u, const QString &text, const QString &suggestedFileName, QWidget *widget)
 {
     bool ok;
@@ -88,39 +120,17 @@ static QUrl getNewFileName(const QUrl &u, const QString &text, const QString &su
     QUrl myurl(u);
     myurl.setPath(concatPaths(myurl.path(), file));
 
-    KIO::StatJob *job = KIO::stat(myurl, myurl.isLocalFile() ? KIO::HideProgressInfo : KIO::DefaultFlags);
-    job->setDetails(0);
-    job->setSide(KIO::StatJob::DestinationSide);
-    KJobWidgets::setWindow(job, widget);
-
-    // Check for existing destination file.
-    // When we were using CopyJob, we couldn't let it do that (would expose
-    // an ugly tempfile name as the source URL)
-    // And now we're using a put job anyway, no destination checking included.
-    if (job->exec()) {
-        //qDebug() << "Paste will overwrite file.  Prompting...";
-        KIO::RenameDialog dlg(widget,
-                              i18n("File Already Exists"),
-                              u,
-                              myurl,
-                              KIO::RenameDialog_Overwrite);
-        KIO::RenameDialog_Result res = static_cast<KIO::RenameDialog_Result>(dlg.exec());
-
-        if (res == KIO::Result_Rename) {
-            myurl = dlg.newDestUrl();
-        } else if (res == KIO::Result_Cancel) {
-            return QUrl();
-        } else if (res == KIO::Result_Overwrite) {
-            // OK, proceed
-        }
-    }
-
-    return myurl;
+    return getDestinationUrl(u, myurl, widget);
 }
 
 static KIO::Job *putDataAsyncTo(const QUrl &url, const QByteArray &data, QWidget *widget, KIO::JobFlags flags)
 {
     KIO::Job *job = KIO::storedPut(data, url, -1, flags);
+    QObject::connect(job, &KIO::Job::result, [url](KJob *job) {
+        if (job->error() == KJob::NoError) {
+            org::kde::KDirNotify::emitFilesAdded(url.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
+        }
+    });
     KJobWidgets::setWindow(job, widget);
     return job;
 }
@@ -135,6 +145,7 @@ static QByteArray chooseFormatAndUrl(const QUrl &u, const QMimeData *mimeData,
 {
     QMimeDatabase db;
     QStringList formatLabels;
+    formatLabels.reserve(formats.size());
     for (int i = 0; i < formats.size(); ++i) {
         const QString &fmt = formats[i];
         QMimeType mime = db.mimeTypeForName(fmt);
@@ -149,8 +160,8 @@ static QByteArray chooseFormatAndUrl(const QUrl &u, const QMimeData *mimeData,
     if (dialogText.isEmpty()) {
         dialogText = i18n("Filename for clipboard content:");
     }
-    //using QString() instead of QString::null didn't compile (with gcc 3.2.3), because the ctor was mistaken as a function declaration, Alex //krazy:exclude=nullstrassign
-    KIO::PasteDialog dlg(QString::null, dialogText, suggestedFileName, formatLabels, widget, clipboard);   //krazy:exclude=nullstrassign
+
+    KIO::PasteDialog dlg(QString(), dialogText, suggestedFileName, formatLabels, widget, clipboard);
 
     if (dlg.exec() != QDialog::Accepted) {
         return QByteArray();
@@ -170,6 +181,10 @@ static QByteArray chooseFormatAndUrl(const QUrl &u, const QMimeData *mimeData,
     //qDebug() << " result=" << result << " chosenFormat=" << chosenFormat;
     *newUrl = u;
     newUrl->setPath(concatPaths(newUrl->path(), result));
+
+    const QUrl destUrl = getDestinationUrl(u, *newUrl, widget);
+    *newUrl = destUrl;
+
     // In Qt3, the result of clipboard()->mimeData() only existed until the next
     // event loop run (see dlg.exec() above), so we re-fetched it.
     // TODO: This should not be necessary with Qt5; remove this conditional
@@ -232,7 +247,7 @@ KIO::Job *pasteMimeDataImpl(const QMimeData *mimeData, const QUrl &destUrl,
         } else if (formats.size() > 1) {
             QUrl newUrl;
             ba = chooseFormatAndUrl(destUrl, mimeData, formats, dialogText, suggestedFilename, widget, clipboard, &newUrl);
-            if (ba.isEmpty()) {
+            if (ba.isEmpty() || newUrl.isEmpty()) {
                 return nullptr;
             }
             return putDataAsyncTo(newUrl, ba, widget, KIO::Overwrite);
